@@ -5,5 +5,6 @@ from lxml.builder import E
 
 
 def facts_session(dev, facts):
-    facts['HOME'] = dev.rpc(
-        E.command("show cli directory")).findtext('./working-directory')
+    facts["HOME"] = dev.rpc(E.command("show cli directory")).findtext(
+        "./working-directory"
+    )
