@@ -3,5 +3,6 @@ Pythonifier for Static route Table/View
 """
 from jnpr.junos.factory import loadyaml
 from os.path import splitext
-_YAML_ = splitext(__file__)[0] + '.yml'
+
+_YAML_ = splitext(__file__)[0] + ".yml"
 globals().update(loadyaml(_YAML_))
